/*==============================================
   This file is a part of the QENet library.
         Copyright 2012 - Nathan Osman
==============================================*/

#include <QENetHost>

#include "QENetAddressPrivate.h"
#include "QENetHostPrivate.h"
#include "QENetPeerPrivate.h"

QENetHostPrivate::QENetHostPrivate()
    : host(NULL)
{
}

QENetHost::QENetHost(QObject * parent)
    : QObject(parent), d(new QENetHostPrivate)
{
    connect(&d->timer, SIGNAL(timeout()), SLOT(service()));
}

QENetHost::~QENetHost()
{
    close();
    delete d;
}

bool QENetHost::create(quint32 channel_limit, quint32 num_peers, int service_interval)
{
    return create(QENetAddress(), channel_limit, num_peers, service_interval);
}

bool QENetHost::create(const QENetAddress & address, quint32 channel_limit, quint32 num_peers, int service_interval)
{
    Q_ASSERT(!d->host);

    if((d->host = enet_host_create((address.d->initialized)?(&address.d->address):NULL,
                                   num_peers, channel_limit, 0, 0)))
    {
        if(service_interval)
            d->timer.start(service_interval);

        return true;
    }

    return false;
}

bool QENetHost::connectToPeer(const QENetAddress & address, quint32 num_channels, quint32 data)
{
    Q_ASSERT(d->host);

    return (enet_host_connect(d->host, &address.d->address, num_channels, data));
}

void QENetHost::close()
{
    if(d->host)
    {
        d->timer.stop();
        enet_host_destroy(d->host);
        d->host = NULL;
    }
}

void QENetHost::broadcast(quint32 channel, const QENetPacket & packet) const
{
    Q_ASSERT(d->host);

    ENetPacket * p = enet_packet_create(qPrintable(packet.data()), packet.data().length(), packet.flags());
    enet_host_broadcast(d->host, channel, p);
}

void QENetHost::flush() const
{
    Q_ASSERT(d->host);

    enet_host_flush(d->host);
}

void QENetHost::setChannelLimit(quint32 channel_limit)
{
    Q_ASSERT(d->host);

    enet_host_channel_limit(d->host, channel_limit);
}

void QENetHost::setBandwidthLimit(quint32 incoming, quint32 outgoing)
{
    Q_ASSERT(d->host);

    enet_host_bandwidth_limit(d->host, incoming, outgoing);
}

void QENetHost::service()
{
    Q_ASSERT(d->host);

    ENetEvent event;
    while(enet_host_service(d->host, &event, 0) > 0)
    {
        switch(event.type)
        {
            case ENET_EVENT_TYPE_CONNECT:
            {
                QENetPeer * peer = new QENetPeer(this);
                peer->d->peer = event.peer;
                peer->d->peer->data = peer;
                emit peerConnected(peer, event.data);
                break;
            }
            case ENET_EVENT_TYPE_RECEIVE:
            {
                QByteArray data(reinterpret_cast<const char *>(event.packet->data), event.packet->dataLength);
                static_cast<QENetPeer *>(event.peer->data)->received(QENetPacket(data, event.packet->flags),
                                                                     event.channelID);
                break;
            }
            case ENET_EVENT_TYPE_DISCONNECT:
            {
                QENetPeer * peer = static_cast<QENetPeer *>(event.peer->data);
                if(peer)
                {
                    peer->d->peer = NULL;
                    peer->disconnected(event.data);
                }
                else
                    emit connectionFailed();
                break;
            }
            default:
                qWarning("Unknown event type received.");
                break;
        }
    }
}
