/*==============================================
   This file is a part of the QENet library.
         Copyright 2012 - Nathan Osman
==============================================*/

#include <enet/enet.h>
#include <QtTest/QtTest>
#include <QENet>
#include <QHostInfo>
#include <QSignalSpy>

class TestQENet : public QObject
{
    Q_OBJECT

    private slots:

        void initTestCase();
        void testQENetAddress();
        void testQENetHostAndQENetPeer();
        void testQENetPacket();

    private:

        QENetInitializer initializer;
};

Q_DECLARE_METATYPE(QENetPeer *)
Q_DECLARE_METATYPE(QENetPacket)

void TestQENet::initTestCase()
{
    qRegisterMetaType<QENetPeer *>();
    qRegisterMetaType<QENetPacket>();
}

void TestQENet::testQENetAddress()
{
    QENetAddress addr;

    QVERIFY(addr.host() == 0);
    QVERIFY(addr.port() == 0);
    QCOMPARE(addr.toString(), QString("0.0.0.0"));

    QVERIFY(addr.resolve("localhost"));

    QVERIFY(addr.host() == 0x0100007f);
    QVERIFY(addr.port() == 0);
    QCOMPARE(addr.toString(), QString("127.0.0.1"));

    addr.setPort(8000);

    QVERIFY(addr.host() == 0x0100007f);
    QVERIFY(addr.port() == 8000);

#ifdef Q_OS_WIN
    QCOMPARE(addr.reverse(), QHostInfo::localHostName());
#else
    QCOMPARE(addr.reverse(), QString("localhost"));
#endif
}

void TestQENet::testQENetHostAndQENetPeer()
{
    QENetHost server;
    QSignalSpy server_peer_connected_spy(&server, SIGNAL(peerConnected(QENetPeer*,quint32)));

    QVERIFY(server.create(QENetAddress(QENetAddress::HostLocal, 8000), 0, QENetHost::DefaultServerPeers, 10));

    QENetHost client;
    QSignalSpy client_peer_connected_spy(&client, SIGNAL(peerConnected(QENetPeer*,quint32)));

    QVERIFY(client.create(0, 1, 10));
    QVERIFY(client.connectToPeer(QENetAddress(QENetAddress::HostLocal, 8000), 12, 12345));

    QTest::qWait(100);

    QCOMPARE(server_peer_connected_spy.count(), 1);
    QCOMPARE(client_peer_connected_spy.count(), 1);
    QCOMPARE(server_peer_connected_spy.at(0).at(1).toInt(), 12345);

    QENetPeer * server_peer = server_peer_connected_spy.at(0).at(0).value<QENetPeer *>();
    QSignalSpy server_peer_received_spy(server_peer, SIGNAL(received(QENetPacket,quint8)));

    QENetPeer * client_peer = client_peer_connected_spy.at(0).at(0).value<QENetPeer *>();

    QVERIFY(client_peer->address().host() == QENetAddress::HostLocal);
    QVERIFY(client_peer->address().port() == 8000);
    QVERIFY(!client_peer->send(QENetPacket("test"), 12));
    QVERIFY(client_peer->send(QENetPacket("test"), 11));

    QTest::qWait(100);

    QCOMPARE(server_peer_received_spy.count(), 1);
    QCOMPARE(server_peer_received_spy.at(0).at(0).value<QENetPacket>().data(), QByteArray("test"));
    QCOMPARE(server_peer_received_spy.at(0).at(1).toInt(), 11);

    QSignalSpy client_peer_disconnected_spy(client_peer, SIGNAL(disconnected(quint32)));
    server_peer->disconnectFromPeer(54321);

    QTest::qWait(100);

    QCOMPARE(client_peer_disconnected_spy.count(), 1);
    QCOMPARE(client_peer_disconnected_spy.at(0).at(0).toInt(), 54321);
}

void TestQENet::testQENetPacket()
{
    QENetPacket packet;

    QCOMPARE(packet.data(), QByteArray(""));
    QVERIFY(packet.flags() == 0);

    packet.setData("test");

    QCOMPARE(packet.data(), QByteArray("test"));
    QVERIFY(packet.flags() == 0);

    packet.setFlags(QENetPacket::Reliable | QENetPacket::Unsequenced);

    QCOMPARE(packet.data(), QByteArray("test"));
    QVERIFY(packet.flags() == (ENET_PACKET_FLAG_RELIABLE | ENET_PACKET_FLAG_UNSEQUENCED));
}

QTEST_MAIN(TestQENet)
#include "TestQENet.moc"
